import os
import pyilz.refresh_token as refresh_token


def get_token(ref_token=None):
    """
    Returns the API_KEY from the environment. If it doesn't exist, refreshes the token and returns it.
    If the refresh token is not provided, it will be retrieved from the environment.

    Args:
        ref_token (str, optional): The refresh token to use. Defaults to None.

    Returns:
        str: The API key.
    """
    API_KEY = os.getenv('API_KEY')
    if API_KEY:
        return API_KEY
    else:
        API_KEY = refresh_token.refresh_token(ref_token)
        os.environ['API_KEY'] = API_KEY
    return API_KEY



from pyilz.get_game_state import get_game_state
from pyilz.parse_land import parse_land
from pyilz.get_plots import get_plot_metadata
import json
import importlib.resources

with importlib.resources.open_text("pyilz", "reference_data.json") as file:
    reference_data = json.load(file)


def nft_to_iz(x, y):
    """
    Converts NFT coordinates to Iz dimensions.

    Args:
        x (int): The x-coordinate of the NFT.
        y (int): The y-coordinate of the NFT.

    Returns:
        tuple: A tuple containing the x and y coordinates in Iz dimensions.
    """
    x = int(x)
    y = int(y)
    num = x - 24
    num2 = -(y-25)
    num3 = -num
    x = num2 + 24
    y = num3 + 25
    return x, y


def get_building_reference_data(building_name):
    """
    Returns the width, height, and image of a building from the reference data.

    Args:
        building_name (str): The name of the building to retrieve data for.

    Returns:
        tuple: A tuple containing the width (int), height (int), and image (str or None) of the building.
    """
    buildings = reference_data['buildings']
    if building_name in buildings:
        building = buildings[building_name]
        img = None
        if 'img' in building:
            img = building['img']
        return building['w'], building['h'], img
    else:
        print(building_name, 'not in reference data')
        return 2, 2, None


def metadata_to_array(gamestate):
    """
    Converts gamestate metadata to an array of building data.

    Args:
        gamestate (dict): The gamestate metadata.

    Returns:
        str: A string representation of the array of building data.
    """
    state = parse_land(gamestate)
    output = ""
    for k, v in state.iterrows():
        x, y = nft_to_iz(v['position']['@x'], v['position']['@y'])
        w, h, img = get_building_reference_data(v['buildingTypeString'][:-2])
        x = x-(h-2)
        y = y-(w-2)
        output += f"[\"{v['buildingTypeString']}\",{y},{x}],"
    output = f'[{output[:-1]}]'
    return output


def import_string_to_array(string):
    """
    Converts a string representation of an array of building types and their coordinates
    into a list of dictionaries, where each dictionary represents a building type and its coordinates.

    Args:
    string (str): A string representation of an array of building types and their coordinates.
                  The string should be in the format '[buildingTypeString1, X1, Y1, buildingTypeString2, X2, Y2, ...]'

    Returns:
    list: A list of dictionaries, where each dictionary represents a building type and its coordinates.
          Each dictionary has the keys 'buildingTypeString', 'X', and 'Y'.
    """
    s = string.replace('[', '').replace(']', '').replace(
        '"', '').replace(' ', '').split(',')
    return [{'buildingTypeString': s[i], 'X':int(s[i+1]), 'Y':int(s[i+2])} for i in range(0, len(s), 3)]


if __name__ == '__main__':
    import dotenv
    dotenv.load_dotenv()
    print(metadata_to_array(get_game_state()['data'][1]['data']))



import pandas as pd
import xmltodict


def parse_land(data):
    """
    Parses the given XML data and returns a pandas dataframe of the land.

    Args:
        data (str): The XML data to parse.

    Returns:
        pandas.DataFrame: A dataframe containing the land data.
    """
    data = xmltodict.parse(data)
    building_data = data['SaveGameData']['Buildings']['BuildingData']
    building_data = pd.DataFrame(building_data)
    return building_data


def get_buildings_and_activities(df):
    """
    Returns a tuple of paths, buildings, current activities, auto activities, and completed activities.
    These are all pandas dataframes.

    Args:
        df (pandas.DataFrame): The input dataframe containing information about buildings and activities.

    Returns:
        tuple: A tuple of pandas dataframes containing information about paths, buildings, current activities, auto activities, and completed activities.
    """

    # convert position column from {'@x': '38', '@y': '17'} to '38,17'
    df['position'] = df['position'].apply(lambda x: x['@x'] + ',' + x['@y'])
    # split position column into x and y columns
    df[['x', 'y']] = df['position'].str.split(',', expand=True)
    # set x and y to int
    df['x'] = df['x'].astype(int)
    df['y'] = df['y'].astype(int)
    df = df.drop(columns=['position'])

    paths = df[df['buildingTypeString'] == 'PATH']
    # drop all columns except for position
    paths = paths[['x', 'y']]

    # Drop all rows where buildingTypeString is PATH
    buildings = df[df['buildingTypeString'] != 'PATH']

    if 'currentActivity' in buildings.columns:
        # get rows where currentActivity is not null
        ca = buildings[buildings['currentActivity'].notnull()]

        # unpack json column currentActivity
        current_activity_cols = pd.json_normalize(
            ca['currentActivity']).set_index(ca.index)
        ca = ca.join(current_activity_cols.add_prefix(
            'currentActivity.'), how='outer')
        ca = ca.drop(columns=['currentActivity'])
        # convert currentActivity.StartTime and currentActivity.EndTime to datetime
        ca['currentActivity.StartTime'] = pd.to_datetime(
            ca['currentActivity.StartTime'])
        ca['currentActivity.EndTime'] = pd.to_datetime(
            ca['currentActivity.EndTime'])
        # remove _{number} from currentActivity.Type and put into column currentActivity.Level
        ca['currentActivity.Level'] = ca['currentActivity.Type'].str.extract(
            r'_(\d+)', expand=False).fillna(0).astype(int)
        ca['currentActivity.Type'] = ca['currentActivity.Type'].str.replace(
            r'_(\d+)', '', regex=True)

    else:
        ca = None

    if 'autoActivity' in buildings.columns:
        # get rows where autoActivity is not null
        aa = buildings[buildings['autoActivity'].notnull()]

        # unpack json column autoActivity
        auto_activity_cols = pd.json_normalize(
            aa['autoActivity']).set_index(aa.index)
        aa = aa.join(auto_activity_cols.add_prefix(
            'autoActivity.'), how='outer')
        aa = aa.drop(columns=['autoActivity'])
        # convert autoActivity.StartTime and autoActivity.EndTime to datetime
        aa['autoActivity.StartTime'] = pd.to_datetime(
            aa['autoActivity.StartTime'])
        aa['autoActivity.EndTime'] = pd.to_datetime(aa['autoActivity.EndTime'])
        # remove _{number} from autoActivity.Type and put into column autoActivity.Level
        aa['autoActivity.Level'] = buildings['buildingTypeString'].str.extract(
            r'(\d+)$', expand=False).fillna(0).astype(int)
        aa['autoActivity.Type'] = buildings['buildingTypeString'].str.extract(
            r'(.+?)\d+$', expand=False).str.rstrip('_') + '_AUTOMATIC'
        # aa['autoActivity.GeneratedResources'] = buildings['fractionalGeneratedResources']
    else:
        aa = None

    if 'completedActivity' in buildings.columns:
        # get rows where completedActivity is not null
        completed = buildings[buildings['completedActivity'].notnull()]

        # unpack json column completedActivity
        auto_activity_cols = pd.json_normalize(
            completed['completedActivity']).set_index(completed.index)
        completed = completed.join(auto_activity_cols.add_prefix(
            'completedActivity.'), how='outer')
        completed = completed.drop(columns=['completedActivity'])
        # convert completedActivity.StartTime and completedActivity.EndTime to datetime
        completed['completedActivity.StartTime'] = pd.to_datetime(
            completed['completedActivity.StartTime'])
        completed['completedActivity.EndTime'] = pd.to_datetime(
            completed['completedActivity.EndTime'])
        # remove _{number} from completedActivity.Type and put into column completedActivity.Level
        completed['completedActivity.Level'] = buildings['buildingTypeString'].str.extract(
            r'(\d+)$', expand=False).fillna(0).astype(int)
        completed['completedActivity.Type'] = buildings['buildingTypeString'].str.extract(
            r'(.+?)\d+$', expand=False).str.rstrip('_') + '_AUTOMATIC'
        # completed['completedActivity.GeneratedResources'] = buildings['fractionalGeneratedResources']
    else:
        completed = None

    # drop columns uid, state, startTime
    buildings = buildings.drop(columns=[
        'uid', 'state', 'startTime',
    ])

    # convert generatedResources to int
    if 'generatedResources' in buildings.columns:
        buildings['generatedResources'] = buildings['generatedResources'].astype(
            int)

    # create column that is the last number of the buildingTypeString
    buildings['level'] = buildings['buildingTypeString'].str.extract(r'(\d+)$',
                                                                     expand=False)

    buildings['name'] = buildings['buildingTypeString'].str.extract(
        r'(.+?)\d+$', expand=False).str.rstrip('_')

    # if name is nan set name to buildingTypeString
    buildings['name'] = buildings['name'].fillna(
        buildings['buildingTypeString'])
    buildings['level'] = buildings['level'].fillna(0)

    buildings = buildings.drop(columns=['buildingTypeString'])

    return paths, buildings, ca, aa, completed



import math
from pyilz.get_buildings import get_building_dimensions, get_building_starting_efficiency, get_building_radius
from pyilz.metadata_to_array import import_string_to_array


def calculate_efficiency(building_a_pos, building_a_wxh, building_b_pos, building_b_wxh, base_efficiency=80, power=20, radius=7, debuff=False):
    """
    Calculates the efficiency of a building based on its position, size, and distance from another building.

    Args:
        building_a_pos (tuple): The position of the first building as a tuple of (x, y) coordinates.
        building_a_wxh (tuple): The width and height of the first building as a tuple of (width, height).
        building_b_pos (tuple): The position of the second building as a tuple of (x, y) coordinates.
        building_b_wxh (tuple): The width and height of the second building as a tuple of (width, height).
        base_efficiency (int, optional): The base efficiency of the building. Defaults to 80.
        power (int, optional): The power of the building. Defaults to 20.
        radius (int, optional): The radius of the building. Defaults to 7.
        debuff (bool, optional): Whether the building has a debuff. Defaults to False.

    Returns:
        int: The efficiency of the building as an integer.
    """
    if power < 0:
        debuff = True
    building_a_shape = [(i, j) for i in range(building_a_wxh[0])
                        for j in range(building_a_wxh[1])]
    building_b_shape = [(i, j) for i in range(building_b_wxh[0])
                        for j in range(building_b_wxh[1])]
    dist_min = get_minimum_distance_between_buildings(
        building_a_pos, building_a_shape, building_b_pos, building_b_shape, radius)

    if dist_min is None:
        return base_efficiency

    radius_diagonal = math.dist((0, 0), (radius, radius))
    dist_factor = power / radius_diagonal

    efficiency = power - (dist_min - 1) * dist_factor
    if not debuff and efficiency > power:
        efficiency = power
    efficiency += base_efficiency

    if not debuff and efficiency < base_efficiency:
        return base_efficiency
    return efficiency


def get_minimum_distance_between_buildings(building_a_pos, building_a_shape, building_b_pos, building_b_shape, radius=6):
    """
    Calculates the minimum distance between two buildings given their positions and shapes.

    Args:
        building_a_pos (tuple): The position of the first building as a tuple of (x, y) coordinates.
        building_a_shape (list): A list of tuples representing the shape of the first building.
        building_b_pos (tuple): The position of the second building as a tuple of (x, y) coordinates.
        building_b_shape (list): A list of tuples representing the shape of the second building.
        radius (int, optional): The radius within which to search for points. Defaults to 6.

    Returns:
        float: The minimum distance between the two buildings, or None if the minimum points are not within the radius of building_a.
    """
    # Pre-calculate shape positions
    a_positions = get_building_points(building_a_pos, building_a_shape)
    b_positions = get_building_points(building_b_pos, building_b_shape)

    # Initialize variables
    min_distance = float('inf')
    min_a = (0, 0)
    min_b = (0, 0)

    # Calculate all points within radius for both buildings
    p_a = get_points_in_radius(building_a_pos, building_a_shape, radius)
    p_b = get_points_in_radius(building_b_pos, building_b_shape, radius)

    # Find minimum distance between buildings
    for a in a_positions:
        for b in b_positions:
            distance = math.dist(a, b)
            if distance < min_distance and a != b:
                min_distance = distance
                min_a = a
                min_b = b

    # print(f'\t\t{min_distance} {min_a} {min_b}')
    # Check if minimum points are within radius of building_a
    if min_a in p_a and min_b in p_b and min_b in p_a:
        return min_distance
    return None


def get_points_in_radius(origin, shape, radius=6):
    """
    Returns a list of unique points within a given radius of an origin point, 
    taking into account the shape of the object being measured.

    Args:
    - origin (tuple): A tuple containing the x and y coordinates of the origin point.
    - shape (list): A list of tuples containing the x and y coordinates of each point in the shape.
    - radius (int): The radius within which to search for points. Default is 6.

    Returns:
    - list: A list of unique points within the given radius of the origin point.
    """
    x0, y0 = origin
    points = set()
    for dx in range(-radius, radius+1):
        for dy in range(-radius, radius+1):
            for x, y in shape:
                point = (x0 + x + dx, y0 + y + dy)
                points.add(point)
    return list(points)


def get_building_points(building_pos, building_shape):
    """
    Returns a list of points representing the building's shape, given its position and shape.

    Args:
        building_pos (tuple): The (x, y) position of the building.
        building_shape (list): A list of (x, y) tuples representing the shape of the building.

    Returns:
        list: A list of (x, y) tuples representing the points that make up the building's shape.
    """
    x0, y0 = building_pos
    points = [(x0 + x, y0 + y) for x, y in building_shape]
    return points


# define the influence values for each building combination
# anything not defined is assumed 0
influence_values = {
    ('CONDENSER_PLANT', 'HYDROGEN_PUMP'): 30,  # MATCHING RESOURCE
    ('CONDENSER_PLANT', 'LAKE'): 30,  # MATCHING RESOURCE
    ('CONDENSER_PLANT', 'POWER_STATION'): 20,  # POWER
    ('CONDENSER_PLANT', 'SEQUESTRIAN_PLANT'): -10,  # OTHER CONVERTER
    ('CONDENSER_PLANT', 'PHOTODISINTEGRATION_PLANT'): -10,  # OTHER CONVERTER
    ('CONDENSER_PLANT', 'CONDENSER_PLANT'): -40,  # SAME BUILDING

    ('SEQUESTRIAN_PLANT', 'SEDIMENT_EXCAVATOR'): 30,  # MATCHING RESOURCE
    ('SEQUESTRIAN_PLANT', 'OUTCROP'): 30,  # MATCHING RESOURCE
    ('SEQUESTRIAN_PLANT', 'POWER_STATION'): 20,  # POWER
    ('SEQUESTRIAN_PLANT', 'PHOTODISINTEGRATION_PLANT'): -10,  # OTHER CONVERTER
    ('SEQUESTRIAN_PLANT', 'CONDENSER_PLANT'): -10,  # OTHER CONVERTER
    ('SEQUESTRIAN_PLANT', 'SEQUESTRIAN_PLANT'): -40,  # BUILDING

    ('PHOTODISINTEGRATION_PLANT', 'MINE'): 30,  # MATCHING RESOURCE
    ('PHOTODISINTEGRATION_PLANT', 'SILICON_MOUND'): 30,  # MATCHING RESOURCE
    ('PHOTODISINTEGRATION_PLANT', 'POWER_STATION'): 20,  # POWER
    ('PHOTODISINTEGRATION_PLANT', 'SEQUESTRIAN_PLANT'): -10,  # OTHER CONVERTER
    ('PHOTODISINTEGRATION_PLANT', 'CONDENSER_PLANT'): -10,  # OTHER CONVERTER
    ('PHOTODISINTEGRATION_PLANT', 'PHOTODISINTEGRATION_PLANT'): -40,  # BUILDING

    ('ANTI-SOLON_INVERTER', 'SOLON_DREDGE'): 30,  # MATCHING RESOURCE
    ('ANTI-SOLON_INVERTER', 'SOLON_TRENCH'): 30,  # MATCHING RESOURCE
    ('ANTI-SOLON_INVERTER', 'POWER_STATION'): 10,  # POWER
    ('ANTI-SOLON_INVERTER', 'L-CRYPTON_COLLIDER'): -30,  # OTHER CONVERTER
    ('ANTI-SOLON_INVERTER', 'HYPERION_LATHE'): -30,  # OTHER CONVERTER
    ('ANTI-SOLON_INVERTER', 'ANTI-SOLON_INVERTER'): -60,  # SAME BUILDING

    ('L-CRYPTON_COLLIDER', 'CRYPTON_GATE'): 30,  # MATCHING RESOURCE
    ('L-CRYPTON_COLLIDER', 'CRYPTON_RIFT'): 30,  # MATCHING RESOURCE
    ('L-CRYPTON_COLLIDER', 'POWER_STATION'): 10,  # POWER
    ('L-CRYPTON_COLLIDER', 'HYPERION_LATHE'): -30,  # OTHER CONVERTER
    ('L-CRYPTON_COLLIDER', 'ANTI-SOLON_INVERTER'): -30,  # OTHER CONVERTER
    ('L-CRYPTON_COLLIDER', 'L-CRYPTON_COLLIDER'): -60,  # SAME BUILDING

    ('HYPERION_LATHE', 'HYPERION_RESERVOIR'): 30,  # MATCHING RESOURCE
    ('HYPERION_LATHE', 'HYPERION_EXTRACTOR'): 30,  # MATCHING RESOURCE
    ('HYPERION_LATHE', 'POWER_STATION'): 10,  # POWER
    ('HYPERION_LATHE', 'ANTI-SOLON_INVERTER'): -30,  # OTHER CONVERTER
    ('HYPERION_LATHE', 'L-CRYPTON_COLLIDER'): -30,  # OTHER CONVERTER
    ('HYPERION_LATHE', 'HYPERION_LATHE'): -60,  # SAME BUILDING

    ('MARKETPLACE', 'POWER_STATION'): 20,

    ('HYDROGEN_PUMP', 'POWER_STATION'): 20,  # POWER
    ('SEDIMENT_EXCAVATOR', 'POWER_STATION'): 20,  # POWER
    ('MINE', 'POWER_STATION'): 20,  # POWER

    ('MATERIALS_LAB', 'DATA_BANK'): 30,
    ('MATERIALS_LAB', 'POWER_STATION'): 20,
    ('MATERIALS_LAB', 'MATERIALS_LAB'): -10,

    ('ZERO_POINT_TRANSDUCER', 'POWER_STATION'): 20,
    ('SINGULARITY_SCANNER', '*'): -10,
}

# get the other building's influence on this building


def get_power_influence(this, other):
    """
    Calculates the power influence between two buildings based on their building types.

    Args:
        this (dict): A dictionary containing information about the first building.
        other (dict): A dictionary containing information about the second building.

    Returns:
        float: The power influence between the two buildings.
    """
    inf = influence_values.get(
        (this['buildingTypeString'][:-2], other['buildingTypeString'][:-2]), None)
    if not inf:
        inf = influence_values.get((this['buildingTypeString'][:-2], '*'), 0)
    return inf


def compute(import_string, printing=False):
    """
    Computes the efficiency of each building in the import_string based on its power influence and proximity to other buildings.

    Args:
        import_string (list): A list of dictionaries containing information about each building.
        printing (bool, optional): Whether to print the intermediate steps of the computation. Defaults to False.

    Returns:
        list: A list of dictionaries containing the name and efficiency of each building.
    """
    IGNORE_LIST = ['POWER_STATION', 'HYDROGEN_MATTER_SILO', 'ENGINEERING_WORKSHOP',
                   'NEXUS', 'SILICON_MATTER_SILO', 'QUANTUM_FABRICANT', 'CARBON_MATTER_SILO']
    IGNORE_LIST_2 = ['HYDROGEN_MATTER_SILO', 'ENGINEERING_WORKSHOP',
                     'NEXUS', 'SILICON_MATTER_SILO', 'QUANTUM_FABRICANT', 'CARBON_MATTER_SILO']
    output = []
    for building in import_string:
        name = building['buildingTypeString']
        if name[:-2] in IGNORE_LIST:
            continue
        w, h = get_building_dimensions(name[:-2])
        wxh = (w, h)
        eff = default_efficiency = get_building_starting_efficiency(name[:-2])
        radius = get_building_radius(name[:-2])
        if printing and name != 'POWER_STATION_1':
            print(f'--- {name} - {eff} -{w}x{h}---')
        for building2 in import_string:
            name2 = building2['buildingTypeString']
            if name2[:-2] in IGNORE_LIST_2:
                continue
            w2, h2 = get_building_dimensions(name2[:-2])
            wxh2 = (w2, h2)
            if building != building2:
                infl = get_power_influence(building, building2)
                diff = calculate_efficiency((building['X'], building['Y']), wxh,
                                            (building2['X'], building2['Y']), wxh2, base_efficiency=default_efficiency, power=infl, radius=radius)-default_efficiency
                eff += diff
                if printing and diff > 0:
                    print(f'\t{diff} from {name2}')
        if eff > 150:
            eff = 150
        output.append({
            'name': name,
            'efficiency': eff
        })
        if printing and name != 'POWER_STATION_1':
            print(f'Final Efficiency: {eff:.0f}\n')
    return output


if __name__ == '__main__':
    import_string = import_string_to_array(
        '[["SOLON_CONTAINMENT_UNIT_1",35,18],["POWER_STATION_1",10,16],["HYDROGEN_MATTER_SILO_5",27,5],["HYDROGEN_PUMP_5",31,31],["MINE_5",35,27],["SEDIMENT_EXCAVATOR_4",35,31],["NEXUS_7",42,22],["ENGINEERING_WORKSHOP_5",25,11],["HYDROGEN_MATTER_SILO_5",24,2],["CARBON_MATTER_SILO_5",24,5],["PHOTODISINTEGRATION_PLANT_4",35,23],["CARBON_MATTER_SILO_5",24,8],["POWER_STATION_1",2,22],["SILICON_MATTER_SILO_5",21,8],["SEQUESTRIAN_PLANT_5",9,28],["CONDENSER_PLANT_4",27,39],["QUANTUM_FABRICANT_4",22,11],["POWER_STATION_1",15,22],["POWER_STATION_1",6,29],["CARBON_MATTER_SILO_5",32,11],["POWER_STATION_1",32,28],["SILICON_MATTER_SILO_5",29,11],["SEQUESTRIAN_PLANT_4",34,34],["POWER_STATION_1",13,32],["POWER_STATION_1",13,22],["L-CRYPTON_COLLIDER_3",9,19],["CONDENSER_PLANT_5",5,21],["PHOTODISINTEGRATION_PLANT_5",13,25],["POWER_STATION_1",32,26],["POWER_STATION_1",9,22],["POWER_STATION_1",11,22],["POWER_STATION_1",6,18],["POWER_STATION_1",11,32],["CARBON_MATTER_SILO_5",27,8],["HYDROGEN_MATTER_SILO_5",30,8],["ANTI-SOLON_INVERTER_2",38,31],["L-CRYPTON_COLLIDER_2",28,28],["CARBON_MATTER_SILO_5",18,8],["SINGULARITY_SCANNER_1",23,45],["CARBON_MATTER_SILO_5",21,5],["CARBON_MATTER_SILO_5",19,11],["CONDENSER_PLANT_5",13,18],["CRYPTON_CONTAINMENT_UNIT_2",35,14],["HYPERION_CONTAINMENT_UNIT_2",39,18],["SEQUESTRIAN_PLANT_4",31,22],["CARBON_MATTER_SILO_5",16,11],["POWER_STATION_1",28,25],["MATERIALS_LAB_1",13,29],["HYPERION_RESERVOIR_1",9,25],["HYPERION_LATHE_1",6,25]]')
    compute(import_string, printing=True)
    import timeit
    print('Original Method Finished:', timeit.timeit('compute(import_string, printing=True)',
                                                     setup='from __main__ import compute, import_string', number=1))



import requests
import pyilz.get_device_id as get_device_id
import pyilz.refresh_token as refresh_token
import pyilz.get_token as get_token
import os


def get_game_state(api_key=None, device_id=None):
    """
    Returns a dictionary of the game state. Auth required.

    Args:
        api_key (str): API key for authentication. If None, it will be fetched using get_token.get_token().
        device_id (str): Device ID for authentication. If None, it will be fetched using get_device_id.get_device_id().

    Returns:
        dict: A dictionary containing the game state.
    """
    DEVICE_ID = get_device_id.get_device_id() if device_id is None else device_id
    API_KEY = get_token.get_token() if api_key is None else api_key
    url = f'http://api.illuvium-game.io/gamedata/api/zero/gamestate?active_device_id={DEVICE_ID}'

    response = requests.get(url, headers={'Authorization':
                            'Bearer ' + API_KEY})
    if response.status_code == 401:
        API_KEY = refresh_token.refresh_token()
        response = requests.get(url, headers={'Authorization':
                                              'Bearer ' + API_KEY})
        os.environ['API_KEY'] = API_KEY

    return response.json()



try:
    import wmi
    import hashlib
    import pythoncom

    def get_device_id():
        '''
        Returns the unique device ID for this computer the same way the Unity game engine does. This allows us to interact with the game state without getting logged out.

        Currently only works on Windows.

        Returns:
            str: A SHA1 hash of the concatenated serial numbers of the Win32_BaseBoard, Win32_BIOS, and Win32_OperatingSystem WMI classes.
        '''
        # Use CoInitialize to avoid errors like this:
        #    http://stackoverflow.com/questions/14428707/python-function-is-unable-to-run-in-new-thread
        pythoncom.CoInitialize()
        c = wmi.WMI()
        s = ""
        for item in c.query("SELECT * FROM Win32_BaseBoard"):
            s += item.SerialNumber
        for item in c.query("SELECT * FROM Win32_BIOS"):
            s += item.SerialNumber
        for item in c.query("SELECT * FROM Win32_OperatingSystem"):
            s += item.SerialNumber
        return hashlib.sha1(s.encode('utf-8')).hexdigest()
except ImportError:
    def get_device_id():
        return "default_device_id"


if __name__ == "__main__":
    print(get_device_id())
